@@ -1,6 +1,6 @@
 #!/bin/sh
 
-# tst_roundtrip.sh - test that pskctool generates what it parses
+# tst_roundtrip.sh - test that pskctool generates what it parses without errors
 # Copyright (C) 2012 Simon Josefsson
 
 # This program is free software: you can redistribute it and/or modify
@@ -22,11 +22,16 @@ PSKCTOOL=../pskctool
 
 srcdir="${srcdir:-.}"
 pskc_all="$srcdir/pskc-all.xml"
+pskc_all_human="$srcdir/pskc-all-human.txt"
 
-$PSKCTOOL -v -q -d -c $pskc_all > tmp
+$PSKCTOOL --info --verbose --quiet --debug $pskc_all > tmp 2>&1
 
 diff -ur $pskc_all tmp
 
+$PSKCTOOL --info --debug $pskc_all > tmp 2>&1
+
+diff -ur $pskc_all_human tmp
+
 rm -f tmp
 
 exit 0
